#include <stdio.h>
#include <string.h>
#include <ctype.h>


/*
 * An implementation of strstr() with case-insensitivity.
 */
char *
strcasestr(const char *haystack, const char *needle)
{
	char *h, *n, *c;
	size_t hl, nl;

	c = (char *)haystack;
	n = (char *)needle;
	hl = strlen(haystack);
	nl = strlen(needle);

	while (hl >= nl) {
		while (tolower(*c) != tolower(*needle)) {
			c++;
			hl--;
			if (hl < nl)
				return NULL;
		}

		h = c;
		n = (char *)needle;

		while (tolower(*h) == tolower(*n)) {
			h++;
			n++;

			if (*n == '\0')
				return c;
		}
		c++;
		hl--;
	}

	return NULL;
}


/*
 * Convert an unsigned long integer to a string.
 */
char *
ultostr(unsigned long int num, int base)
{
	unsigned int d;
	static char s[65];
	char *c;

	if (base < 2 || base > 36)
		return NULL;

	c = s + 64;
	*c = '\0';

	do {
		d = num % base;
		num /= base;

		if (d <= 9)
			*--c = '0' + d;
		else
			*--c = 'a' + d - 10;
	} while (c >= s && num != 0);

	return c;
}


/*
 * Copies at most size characters from the string pointed by src to the
 * array pointed by dest, always NULL terminating (unless size == 0).
 * Returns pointer to dest.
 */
char *
xstrncpy(char *dest, const char *src, size_t size)
{
	char *d;
	const char *s;
	size_t n;

	d = dest;
	s = src;
	n = size;

	while (n != 0) {
		if ((*d++ = *s++) == '\0')
			break;
		n--;
	}

	if (n == 0 && size != 0)
		*d = '\0';

	return dest;
}
